------------------------------------------------------------------------------
--	FILE:	 FantasyWrapX.lua
--	AUTHOR:  EwokBaconBits
------------------------------------------------------------------------------
--	Copyright (c) 2014 Firaxis Games, Inc. All rights reserved.
------------------------------------------------------------------------------

include "MapEnums"
include "MapUtilities"
include "MountainsCliffs"
include "RiversLakes"
include "FeatureGenerator"
include "TerrainGenerator"
include "NaturalWonderGenerator"
include "ResourceGenerator"
include "AssignStartingPlots"

local g_iW, g_iH;
local g_iFlags = {};
local g_continentsFrac = nil;


-------------------------------------------------------------------------------
function GenerateMap()
	print("Generating Fractal Map");
	local pPlot;

	-- Set globals
	g_iW, g_iH = Map.GetGridSize();
	g_iFlags = TerrainBuilder.GetFractalFlags();

	local temperature = MapConfiguration.GetValue("temperature"); -- Default setting is Temperate.
	if temperature == 4 then
		temperature  =  1 + TerrainBuilder.GetRandomNumber(3, "Random Temperature- Lua");
	end
	
	plotTypes = GeneratePlotTypes();
	terrainTypes = GenerateTerrainTypes(plotTypes, g_iW, g_iH, g_iFlags, false, temperature);


	for i = 0, (g_iW * g_iH) - 1, 1 do
		pPlot = Map.GetPlotByIndex(i);
		if (plotTypes[i] == g_PLOT_TYPE_HILLS) then
			terrainTypes[i] = terrainTypes[i] + 1;
		end
		TerrainBuilder.SetTerrainType(pPlot, terrainTypes[i]);
	end
	
	-- Temp
	AreaBuilder.Recalculate();
	local biggest_area = Areas.FindBiggestArea(false);
	print("After Adding Hills: ", biggest_area:GetPlotCount());

	-- River generation is affected by plot types, originating from highlands and preferring to traverse lowlands.
	AddRivers();
	
	-- Lakes would interfere with rivers, causing them to stop and not reach the ocean, if placed any sooner.
	local numLargeLakes = GameInfo.Maps[Map.GetMapSize()].Continents
	AddLakes(numLargeLakes);

	AddFeatures();
	
	print("Adding cliffs");
	AddCliffs(plotTypes, terrainTypes);

	local args = {
		numberToPlace = GameInfo.Maps[Map.GetMapSize()].NumNaturalWonders,
	};
	local nwGen = NaturalWonderGenerator.Create(args);

	AreaBuilder.Recalculate();
	TerrainBuilder.AnalyzeChokepoints();
	TerrainBuilder.StampContinents();
	
	--for i = 0, (g_iW * g_iH) - 1, 1 do
		--pPlot = Map.GetPlotByIndex(i);
		--print ("i: plotType, terrainType, featureType: " .. tostring(i) .. ": " .. tostring(plotTypes[i]) .. ", " .. tostring(terrainTypes[i]) .. ", " .. tostring(pPlot:GetFeatureType(i)));
	--end
	local resourcesConfig = MapConfiguration.GetValue("resources");
	local args = {
		iWaterLux = 2,
		resources = resourcesConfig,
	};
	local resGen = ResourceGenerator.Create(args);

	print("Creating start plot database.");
	local startconfig = MapConfiguration.GetValue("start"); -- Get the start config
	-- START_MIN_Y and START_MAX_Y is the percent of the map ignored for major civs' starting positions.
	local args = {
		MIN_MAJOR_CIV_FERTILITY = 175,
		MIN_MINOR_CIV_FERTILITY = 50, 
		MIN_BARBARIAN_FERTILITY = 1,
		START_MIN_Y = 15,
		START_MAX_Y = 15,
		START_CONFIG = startconfig,
	};
	local start_plot_database = AssignStartingPlots.Create(args)

	local GoodyGen = AddGoodies(g_iW, g_iH);
	
	-- Place Grassland with forest check for hills don't do forests on mountains
	-- Depending on size of map with forests and count
	
	-- local random_forestY = math.random( 1, g_iH - 1 );
	-- local random_forestX = math.random( 1, g_iW - 1 );
	
	-- local random_forestY = TerrainBuilder.GetRandomNumber(g_iH - 1, "Random Forest Y");
	-- local random_forestX = TerrainBuilder.GetRandomNumber(g_iW - 1, "Random Forest X");
	-- print(random_forestY);
	-- print(random_forestX);
	-- local forestPlot = Map.GetPlot(random_forestX, random_forestY);
	-- for i = 0, 300 do
		-- print('for looping');
		-- if forestPlot:GetTerrainType() == g_TERRAIN_TYPE_OCEAN or forestPlot:GetTerrainType() == g_TERRAIN_TYPE_COAST or forestPlot:GetContinentType() == -1 then
			-- local random_forestY = TerrainBuilder.GetRandomNumber(g_iH - 1, "Random Forest Y");
			-- local random_forestX = TerrainBuilder.GetRandomNumber(g_iW - 1, "Random Forest X");
			-- local forestPlot = Map.GetPlot(random_forestX, random_forestY);
			-- print('NEW PLOT');
		-- else
			-- break;
		-- end
	-- end
	-- while( forestPlot:GetContinentType() == -1 )
	-- do
	    -- local random_forestY = TerrainBuilder.GetRandomNumber(g_iH - 1, "Random Forest Y");
		-- local random_forestX = TerrainBuilder.GetRandomNumber(g_iW - 1, "Random Forest X");
		-- local forestPlot = Map.GetPlot(random_forestX, random_forestY);
		-- if (forestPlot:GetContinentType() ~= -1) then
			-- break;
		-- end
	-- end
	--local forestBiome = forestPlot:GetContinentType();
	local forestCount = 0;
	local forestLimit = (g_iH - 24) * 6;
	local desertCount = 0; --44 106 26 66
	local desertLimit = (g_iH - 20) * 4;
	local random_forestY = TerrainBuilder.GetRandomNumber(g_iH - 15, "Random Forest Y");
	local random_forestX = TerrainBuilder.GetRandomNumber(g_iW - 22, "Random Forest X");
	for x = 0, g_iW - 1 do
		for y = 0, g_iH - 1 do
			local plot = Map.GetPlot(x, y);
			-- LIMIT ICE
			if (plot:GetFeatureType() == g_FEATURE_ICE) then
				if y >= 3 and y <= g_iH - 4 then
					TerrainBuilder.SetFeatureType(plot, -1);
				end
			end
			-- Create a Forest Biome
			-- if forestBiome ~= -1 then
				-- if (plot:GetContinentType() == forestBiome and forestCount <= forestLimit and plot:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and plot:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_MOUNTAIN and plot:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_MOUNTAIN and plot:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_MOUNTAIN and plot:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_MOUNTAIN and plot:GetFeatureType() < 5) then
					-- forestCount = forestCount + 1;
					-- if (plot:GetResourceType() == -1) then
						-- TerrainBuilder.SetFeatureType(plot, -1);
						-- TerrainBuilder.SetFeatureType(plot, g_FEATURE_FOREST);
					-- end
					-- if plot:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and plot:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_HILLS and plot:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_HILLS and plot:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_HILLS and plot:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_HILLS then
						-- TerrainBuilder.SetTerrainType(plot, g_TERRAIN_TYPE_GRASS_HILLS);
					-- else
						-- TerrainBuilder.SetTerrainType(plot, g_TERRAIN_TYPE_GRASS);
					-- end
				-- end
			-- end
			
			if (plot:GetFeatureType() == g_FEATURE_FOREST and forestCount <= forestLimit) then
				fsPlotX = plot:GetX();
				fsPlotY = plot:GetY();
				if fsPlotX ~= 0 and fsPlotY ~= 0 and fsPlotX >= random_forestX and fsPlotY >= random_forestY then
					addPlotX1 = Map.GetPlot(fsPlotX + 1, fsPlotY);
					addPlotX2 = Map.GetPlot(fsPlotX - 1, fsPlotY);
					addPlotY1 = Map.GetPlot(fsPlotX, fsPlotY + 1);
					addPlotY2 = Map.GetPlot(fsPlotX, fsPlotY - 1);
					if addPlotX1:GetFeatureType() ~= g_FEATURE_FOREST and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_HILLS and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_COAST and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_OCEAN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_MOUNTAIN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_MOUNTAIN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_MOUNTAIN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_MOUNTAIN and addPlotX1:GetFeatureType() < 5 and addPlotX1:GetResourceType() == -1 then
						forestCount = forestCount + 1;
						TerrainBuilder.SetFeatureType(addPlotX1, -1);
						TerrainBuilder.SetFeatureType(addPlotX1, g_FEATURE_FOREST);
						if addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_HILLS and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_HILLS and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_HILLS and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_HILLS then
							TerrainBuilder.SetTerrainType(addPlotX1, g_TERRAIN_TYPE_GRASS_HILLS);
						else
							TerrainBuilder.SetTerrainType(addPlotX1, g_TERRAIN_TYPE_GRASS);
						end
					end
					if addPlotX2:GetFeatureType() ~= g_FEATURE_FOREST and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_HILLS and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_COAST and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_OCEAN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_MOUNTAIN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_MOUNTAIN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_MOUNTAIN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_MOUNTAIN and addPlotX2:GetFeatureType() < 5 and addPlotX2:GetResourceType() == -1 then
						forestCount = forestCount + 1;
						TerrainBuilder.SetFeatureType(addPlotX2, -1);
						TerrainBuilder.SetFeatureType(addPlotX2, g_FEATURE_FOREST);
						if addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_HILLS and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_HILLS and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_HILLS and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_HILLS then
							TerrainBuilder.SetTerrainType(addPlotX2, g_TERRAIN_TYPE_GRASS_HILLS);
						else
							TerrainBuilder.SetTerrainType(addPlotX2, g_TERRAIN_TYPE_GRASS);
						end
					end
					if addPlotY1:GetFeatureType() ~= g_FEATURE_FOREST and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_HILLS and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_COAST and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_OCEAN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_MOUNTAIN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_MOUNTAIN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_MOUNTAIN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_MOUNTAIN and addPlotY1:GetFeatureType() < 5 and addPlotY1:GetResourceType() == -1 then
						forestCount = forestCount + 1;
						TerrainBuilder.SetFeatureType(addPlotY1, -1);
						TerrainBuilder.SetFeatureType(addPlotY1, g_FEATURE_FOREST);
						if addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_HILLS and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_HILLS and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_HILLS and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_HILLS then
							TerrainBuilder.SetTerrainType(addPlotY1, g_TERRAIN_TYPE_GRASS_HILLS);
						else
							TerrainBuilder.SetTerrainType(addPlotY1, g_TERRAIN_TYPE_GRASS);
						end
					end
					if addPlotY2:GetFeatureType() ~= g_FEATURE_FOREST and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_HILLS and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_COAST and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_OCEAN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_MOUNTAIN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_MOUNTAIN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_MOUNTAIN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_MOUNTAIN and addPlotY2:GetFeatureType() < 5 and addPlotY2:GetResourceType() == -1 then
						forestCount = forestCount + 1;
						TerrainBuilder.SetFeatureType(addPlotY2, -1);
						TerrainBuilder.SetFeatureType(addPlotY2, g_FEATURE_FOREST);
						if addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_HILLS and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_HILLS and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_HILLS and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_HILLS then
							TerrainBuilder.SetTerrainType(addPlotY2, g_TERRAIN_TYPE_GRASS_HILLS);
						else
							TerrainBuilder.SetTerrainType(addPlotY2, g_TERRAIN_TYPE_GRASS);
						end
					end
				end
			end
			-- Create a Desert Biome
			-- Loop through all deserts do x+1 and y+1 with minus get that tile remove feature (as long as its not a natural wonder or mountain) make it a desert, small chance of an oasis
			if (plot:GetTerrainType() == g_TERRAIN_TYPE_DESERT and desertCount < desertLimit) then
				dsPlotX = plot:GetX();
				dsPlotY = plot:GetY();
				if dsPlotX ~= 0 and dsPlotY ~= 0 then
					addPlotX1 = Map.GetPlot(dsPlotX + 1, dsPlotY);
					addPlotX2 = Map.GetPlot(dsPlotX - 1, dsPlotY);
					addPlotY1 = Map.GetPlot(dsPlotX, dsPlotY + 1);
					addPlotY2 = Map.GetPlot(dsPlotX, dsPlotY - 1);
					if addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_HILLS and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_HILLS and addPlotX1:GetRiverCrossingCount() == 0 and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_COAST and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_OCEAN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_MOUNTAIN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_MOUNTAIN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_MOUNTAIN and addPlotX1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_MOUNTAIN and addPlotX1:GetFeatureType() < 5 then
						desertCount = desertCount + 1;
						TerrainBuilder.SetFeatureType(addPlotX1, -1);
						TerrainBuilder.SetTerrainType(addPlotX1, g_TERRAIN_TYPE_DESERT);
						local random_Oasis = TerrainBuilder.GetRandomNumber(20, "Random Oasis");
						if random_Oasis == 1 and addPlotX1:GetResourceType() == nil then
							TerrainBuilder.SetFeatureType(plot, g_FEATURE_OASIS);
						end
					end
					if addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_HILLS and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_HILLS and addPlotX2:GetRiverCrossingCount() == 0 and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_COAST and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_OCEAN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_MOUNTAIN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_MOUNTAIN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_MOUNTAIN and addPlotX2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_MOUNTAIN and addPlotX2:GetFeatureType() < 5 then
						desertCount = desertCount + 1;
						TerrainBuilder.SetFeatureType(addPlotX2, -1);
						TerrainBuilder.SetTerrainType(addPlotX2, g_TERRAIN_TYPE_DESERT);
						local random_Oasis = TerrainBuilder.GetRandomNumber(20, "Random Oasis");
						if random_Oasis == 1 and addPlotX2:GetResourceType() == nil then
							TerrainBuilder.SetFeatureType(plot, g_FEATURE_OASIS);
						end
					end
					if addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_HILLS and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_HILLS and addPlotY1:GetRiverCrossingCount() == 0 and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_COAST and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_OCEAN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_MOUNTAIN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_MOUNTAIN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_MOUNTAIN and addPlotY1:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_MOUNTAIN and addPlotY1:GetFeatureType() < 5 then
						desertCount = desertCount + 1;
						TerrainBuilder.SetFeatureType(addPlotY1, -1);
						TerrainBuilder.SetTerrainType(addPlotY1, g_TERRAIN_TYPE_DESERT);
						local random_Oasis = TerrainBuilder.GetRandomNumber(20, "Random Oasis");
						if random_Oasis == 1 and addPlotY1:GetResourceType() == nil then
							TerrainBuilder.SetFeatureType(plot, g_FEATURE_OASIS);
						end
					end
					if addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_HILLS and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_HILLS and addPlotY2:GetRiverCrossingCount() == 0 and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_COAST and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_OCEAN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_GRASS_MOUNTAIN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_PLAINS_MOUNTAIN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_TUNDRA_MOUNTAIN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_DESERT_MOUNTAIN and addPlotY2:GetTerrainType() ~= g_TERRAIN_TYPE_SNOW_MOUNTAIN and addPlotY2:GetFeatureType() < 5 then
						desertCount = desertCount + 1;
						TerrainBuilder.SetFeatureType(addPlotY2, -1);
						TerrainBuilder.SetTerrainType(addPlotY2, g_TERRAIN_TYPE_DESERT);
						local random_Oasis = TerrainBuilder.GetRandomNumber(20, "Random Oasis");
						if random_Oasis == 1 and addPlotY2:GetResourceType() == nil then
							TerrainBuilder.SetFeatureType(plot, g_FEATURE_OASIS);
						end
					end
				end
			end
		end
	end
	
end


-------------------------------------------------------------------------------
function GeneratePlotTypes()
	print("Generating Plot Types");
	local plotTypes = {};
	
	local sea_level_low = 58;
	local sea_level_normal = 58;
	local sea_level_high = 58;
	local world_age_new = 5;
	local world_age_normal = 3;
	local world_age_old = 2;
	local extra_mountains = 0;
	local adjust_plates = 1.0;
	local shift_plot_types = true;
	local hills_ridge_flags = g_iFlags;
	local peaks_ridge_flags = g_iFlags;
	local has_center_rift = false;

	--	local world_age
	local world_age = MapConfiguration.GetValue("world_age");
	if (world_age == 1) then
		world_age = world_age_new;
	elseif (world_age == 2) then
		world_age = world_age_normal;
	elseif (world_age == 3) then
		world_age = world_age_old;
	else
		world_age = 2 + TerrainBuilder.GetRandomNumber(4, "Random World Age - Lua");
	end

	--	local sea_level
    local sea_level = MapConfiguration.GetValue("sea_level");
	local water_percent;
	if sea_level == 1 then -- Low Sea Level
		water_percent = sea_level_low
	elseif sea_level == 2 then -- Normal Sea Level
		water_percent =sea_level_normal
	elseif sea_level == 3 then -- High Sea Level
		water_percent = sea_level_high
	else
		water_percent = TerrainBuilder.GetRandomNumber(sea_level_high- sea_level_low, "Random Sea Level - Lua") + sea_level_low  + 1 ;
	end

	-- Set values for hills and mountains according to World Age chosen by user.
	local adjustment = world_age;
	if world_age <= world_age_old  then -- 5 Billion Years
		adjust_plates = adjust_plates * 0.75;
	elseif world_age >= world_age_new then -- 3 Billion Years
		adjust_plates = adjust_plates * 1.5;
	else -- 4 Billion Years
	end

	local polar =  true;

	local fracFlags = {};

	fracFlags.FRAC_POLAR = true;
	local MapSizeTypes = {};
	for row in GameInfo.Maps() do
		MapSizeTypes[row.MapSizeType] = row.PlateValue;
	end
	local sizekey = Map.GetMapSize();
	local numPlates = MapSizeTypes[sizekey] or 4;

	local continent_grain = 3;
	local rift_grain = -1;

	local riftsFrac = Fractal.Create(g_iW, g_iH, rift_grain, {}, 6, 5);
	g_continentsFrac = Fractal.CreateRifts(g_iW, g_iH, continent_grain, fracFlags, riftsFrac, 6, 5);
	g_continentsFrac:BuildRidges(numPlates, {}, 1, 2);
	local iWaterThreshold = g_continentsFrac:GetHeight(water_percent);
	for x = 0, g_iW - 1 do
		for y = 0, g_iH - 1 do
			local i = y * g_iW + x;
			local val = g_continentsFrac:GetHeight(x, y);
			local pPlot = Map.GetPlotByIndex(i);
			if(val <= iWaterThreshold) then
				plotTypes[i] = g_PLOT_TYPE_LAND;
				TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_DESERT);  -- temporary setting so can calculate areas
			else
				plotTypes[i] = g_PLOT_TYPE_OCEAN;
				TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_OCEAN);  -- temporary setting so can calculate areas
			end
		end
	end

	ShiftPlotTypes(plotTypes);
	AreaBuilder.Recalculate();

	local args = {};
	args.world_age = world_age;
	args.iW = g_iW;
	args.iH = g_iH
	args.iFlags = g_iFlags;
	args.tectonic_islands = true;
	args.blendRidge = 15;
	args.blendFract = 1;
	args.extra_mountains = 18;
	mountainRatio = 11 + world_age * 2;
	plotTypes = ApplyTectonics(args, plotTypes);
	plotTypes = AddLonelyMountains(plotTypes, mountainRatio);

	return plotTypes;
end


function GetMapInitData(MapSize)
	local WrapX = false; -- stop the horizontal axis from wrapping i.e. making it bordered

	return {WrapX = WrapX,}
end

----------------------------------------------------------------------------------
function GenerateTerrain()
	
	local terraingen = TerrainGenerator.Create();
	local terrainTypes = terraingen:GenerateTerrain()
		
	SetTerrainTypes(terrainTypes);
end
----------------------------------------------------------------------------------
function AddFeatures()
	
	print("Adding Features");

	-- Get Rainfall setting input by user.
	local rainfall = MapConfiguration.GetValue("rainfall");
	if rainfall == 4 then
		rainfall = 1 + TerrainBuilder.GetRandomNumber(3, "Random Rainfall - Lua");
	end
	
	local args = {rainfall = rainfall}
	local featuregen = FeatureGenerator.Create(args);
	
	featuregen:AddFeatures();
	-- Random Forest
	
	
	
end